!code to take in two lat,lon points and output model values for
!a specified model parameter along the great circle path between
!the two points for the entire radial range of the model
!sampling is defaults to 1 degree along the path and 50 km 
!in depth but can alternatively be specified.
!Output is in degrees along gcp starting at source and increasing towards 
!the receiver, and in depth in km
!------------------------------------------------------
program A3d_gcsect
!------------------------------------------------------

use module_A3d
use def_gparam
use f2kcli !command line interface
implicit none

character(len=20) :: arg
character :: par,par1
real(DP), dimension(:), allocatable :: gclat,gclon
real(DP), parameter :: erad=6371.0
real(DP) :: lat1,lon1,lat2,lon2,ddel,ddep,rcal,plat,plon
real(DP) :: r1,r2,rtot,dep,del,rad,theta,phi,dv,d1,d2
integer :: parn,npar,ndel,ndep,i,j
integer, parameter :: unit_out=357,unit_pole=358,unit_path=359

if(command_argument_count()/=5.and.command_argument_count()/=7) then
   print*,'usage: A3d_gcsect lat1 lon1 lat2 lon2 par path_samp dep_samp'
   print*,'lat and lon values should be in degrees, par should be the'
   print*,'character description of the desired model parameter, and'
   print*,'path_samp and dep_samp are optional arguments specifying'
   print*,'the sampling along path (degrees, default 1) and in'
   print*,'depth (km, default 50).'
   stop
endif


call get_command_argument(1,value=arg)
read(arg,*) lat1
call get_command_argument(2,value=arg)
read(arg,*) lon1
call get_command_argument(3,value=arg)
read(arg,*) lat2
call get_command_argument(4,value=arg)
read(arg,*) lon2
call get_command_argument(5,value=arg)
read(arg,*) par

!default values
ddel=1.0
ddep=50.0

if(command_argument_count()==7) then
   call get_command_argument(6,value=arg)
   read(arg,*) ddel
   call get_command_argument(7,value=arg)
   read(arg,*) ddep
endif

!read model
call init_A3d(0)

!get par number
call get_npar(npar)
parn=0
do i=1,npar
   call get_par(i,par1)
   if(par1==par) parn=i
enddo
if(parn==0) stop 'A3d_slice: invalid par choice'

!determine number of sampling points
ndel=int(360.0/ddel)
allocate(gclat(ndel),gclon(ndel))

!get great-circle path info
call gcp(lat1,lon1,lat2,lon2,ddel,ndel,gclat,gclon,rcal,plat,plon)

!write out pole of gcp
open(unit_pole,file='pole.dat')
write(unit_pole,*) int(plat),int(plon)
close(unit_pole)

!write out gc path
open(unit_path,file='path.dat')
do i=1,ndel
   write(unit_path,*) gclon(i),gclat(i)
enddo
close(unit_path)

!determine depth sampling
call get_rad_range(r1,r2)
rtot=r2-r1
!set first and last depth sample on multiples of ddep
r1=erad-r1
r2=erad-r2
d1=ddep+ddep*int(r2/ddep)
d2=ddep*int(r1/ddep)
ndep=int((d2-d1)/ddep)+1

!write output file
open(unit_out,file='csect.out')

do i=1,ndel
   del=ddel*(i-1)
   theta=deg2rad*(90.0-gclat(i))
   phi=deg2rad*gclon(i)
   do j=1,ndep
      dep=ddep*(j-1)+d1
      rad=erad-dep
      call A3d(rad,theta,phi,parn,dv)
      write(unit_out,101) del,dep,100.0*dv
   enddo
enddo

close(unit_out)

!100 format(f12.4, f12.4)
101 format(f12.4, f12.4, f14.6)
end program A3d_gcsect

!-----------------------------------------------------------------
subroutine gcp(elat1,elon1,slat2,slon2,ddel,ndel,gclat,gclon,rcal,plat,plon)
!-----------------------------------------------------------
!	calculate lats and lons of great circle path
!	note: gclat and gclon are along great circle arc, and start
!	  at event location
!       lifted from /data/07/gung/,bodinvq3/,slice/getsecxyz_global.f
!       on Berkeley Seismo Lab network and updated to f90
!       gclat and gclon are allocated outside this subroutine
  use def_gparam

  real(DP), intent(in) :: elat1,elon1,slat2,slon2,ddel
  integer, intent(in) :: ndel
  real(DP), dimension(ndel) :: gclat,gclon
  real(DP), intent(out) :: rcal,plat,plon

  real(DP) :: zero,the1,phe1,the2,phe2,x1,y1,z1,x2,y2,z2
  real(DP) :: dis,sdis,dis1,px,py,pz,p,a1,a2,a3,b1,b2,b3,x,y,z
  real(DP) :: alat,alon
  integer :: n,i
  !	dimension gclat(500),gclon(500)
  !   	pi=3.14165926
 
  zero=1.0E-8
  !	ddel=1.0
  
  the1=(90.0-elat1)*deg2rad
  phe1=elon1*deg2rad
  the2=(90.0-slat2)*deg2rad
  phe2=slon2*deg2rad
  
  x1=sin(the1)*cos(phe1)
  y1=sin(the1)*sin(phe1)
  z1=cos(the1)
  x2=sin(the2)*cos(phe2)
  y2=sin(the2)*sin(phe2)
  z2=cos(the2)

!	dist between the two points
  dis=acos(x1*x2+y1*y2+z1*z2)*rad2deg
  rcal=dis
  dis=dis*deg2rad
  
!	 the pole of the great circle
  px=y1*z2-y2*z1
  py=x2*z1-x1*z2
  pz=x1*y2-x2*y1
  p=sqrt(px*px+py*py+pz*pz)
     
  if(p.le.zero)then
     print*,'Two same points. Program exits!'
     return
  endif

  px=px/p
  py=py/p
  pz=pz/p
  plat=90.0-acos(pz)*rad2deg
  plon=atan2(py,px)*rad2deg

  sdis=360.
  n=int(sdis/ddel)

!	if(elon1.lt.zero)elon1=360.+elon1
  gcloop: do i=1,n
     if(i.eq.1)then
        dis1=0.0
        gclat(i)=elat1
        gclon(i)=elon1
        if(elon1>180) gclon(i)=elon1-360
        cycle gcloop
     endif
     dis1=float(i-1)* ddel
     dis1=dis1*deg2rad

     
!	   point dis1 degree away from the first point.
!	   use cross product relation of vectors. 
     a1=sin(dis1)/sin(dis-dis1)
     a2=(y1+y2*a1)/(x1+x2*a1)
     a3=(z1+z2*a1)/(x1+x2*a1)
     b1=sin(dis1)/sin(dis)
     b2=a2*x1-y1
     if(b2.le.zero)then
        b3=a3*x1-z1
        x=b1*(x1*z2-x2*z1)/b3
     else
        x=b1*(x1*y2-x2*y1)/b2
     endif
     y=a2*x
     z=a3*x
     alat=90.0-acos(z)*rad2deg
     alon=atan2(y,x)*rad2deg
     dis1=dis1*rad2deg
     gclat(i)=alat
     gclon(i)=alon
  enddo gcloop
!	close(16)
999 return
!-----------------------------------------------------------
end subroutine gcp
!-----------------------------------------------------------

